/*
 * Project name:
     GPS_Test
 * Copyright:
     (c) MikroElektronika, 2005-2010.
  * Revision History:
     20090715:
       - initial release(SZ);
     20101223:
       - added active comments(SZ);
 * Description:
     This is a simple project which demonstrates the use of SmartGM862-GPS accessory board.
     Upon programming the MCU, connect your SmartGM862-GPS to PORTC of your EasyPIC6 development board.
     Be sure to place the GPS antenna OUTDOORS! After your location in the world is found, it will be displayed on the GLCD.
 * Test configuration:
     MCU:             PIC16F887
                      http://ww1.microchip.com/downloads/en/DeviceDoc/41291F.pdf#page=123
     Dev.Board:       EasyPIC6
                      http://www.mikroe.com/eng/products/view/297/easypic6-development-system/
     Oscillator:      HS, 8.00000 MHz
     Ext. Modules:    SmartGM862-GPS on PORTC
                      ac:SmartGM862-GPS
                      http://www.mikroe.com/eng/products/view/469/smartgm862-gps-board/
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/eng/products/view/7/mikroc-pro-for-pic/
 * NOTES:
     - EasyPIC6: J17 - Vcc; PORTD - pull-down
     - Pull-down resistor at RE3 pin
     - Pull-up resistors at RC6 and RC7 pins
     - Upon programming place the GLCD(128x64) on the EasyPIC6 dev. board and turn on the GLCD Back-light(SW6.8-ON)
     - SmartGM862-GPS extra board: enable only SW1.1 and SW1.5 switches
*/

const unsigned short world_bmp[1024];                      // world map image

// GLCD module connections
char GLCD_DataPort at PORTD;

sbit GLCD_CS1 at RB0_bit;    sbit GLCD_CS1_Direction at TRISB0_bit;
sbit GLCD_CS2 at RB1_bit;    sbit GLCD_CS2_Direction at TRISB1_bit;
sbit GLCD_RS  at RB2_bit;    sbit GLCD_RS_Direction  at TRISB2_bit;
sbit GLCD_RW  at RB3_bit;    sbit GLCD_RW_Direction  at TRISB3_bit;
sbit GLCD_EN  at RB4_bit;    sbit GLCD_EN_Direction  at TRISB4_bit;
sbit GLCD_RST at RB5_bit;    sbit GLCD_RST_Direction at TRISB5_bit;
//

// GLCD constants
const char glcd0[] = "POWER UP GPS MODULE!";
const char glcd1[] = "RE0 - Continue!";
const char glcd2[] = "Note: Place the GPS";
const char glcd3[] = "antenna outdoors!";
const char glcd4[] = "Please wait!";
const char glcd5[] = "YOUR LOCATION FOUND!";
//

// AT commands
const char atc0[] = "AT";                                  // every command starts with "AT"
const char atc1[] = "ATE0";                                // disable echo
const char atc2[] = "AT$GPSACP";                           // request for the GPS data
//

// program control variables
char GPS_OK;
char GPS_Data_Ready;
char GPS_Data_Expected;
char GPS_state = 0;
char i,j;
//

//
char GPS_Data[50] absolute 0x20;                           // these addresses are starting addresses in the 0 and 1 RAM banks (16F887)
char GLCD_txt[25] absolute 0xA0;                           // see the datasheet
signed int latitude, longitude;
char tmp = 0;
//

// copy const to ram string
char * CopyConst2Ram(char * dest, const char * src){
char * d ;
 d = dest;
 for(;*dest++ = *src++;)
  ;
 return d;
}

// wait for "OK" response
void wait_GPS_OK() {
  while(!GPS_OK){
  ;
 }
}

// pauses
void wait(){
Delay_ms(1000);
}

void wait2(){
Delay_ms(5000);
}

void wait3(){
Delay_ms(100);
}

// reading the data from UART in the interuppt routine
void interrupt(){                                          // reading the data from UART in the interuppt routine
  if (PIR1.RCIF  == 1){
     tmp = UART1_Read();                                   // get received byte

// process reception through state machine
// we are parsing only the "OK" response
    switch (GPS_state) {

        case 0: {
                 if (tmp == 'O')                           // we have 'O' it might be "OK"
                     GPS_state = 1;
                 else
                     GPS_state = 0;                        // reset state machine
                 break;
                }

        case 1: {
                 if (tmp == 'K')                           // we have 'K'
                     GPS_state = 10;                       // we have "OK" response expecting CR+LF
                 else
                     GPS_state = 0;                        // reset state machine
                 break;
                }

       case 10: {
                if (tmp == 13)                             // we have 13, it could be CR+LF
                    GPS_state = 11;                        // expecting LF
                else
                    GPS_state = 0;                         // reset state machine
                break;
               }

      case 11: {
                if (tmp == 10) {                           // we have LF, response is complete
                    GPS_state = 0;                         // reset state machine
                    GPS_OK = 1;                            // response "OK" is really received!
                }

                break;
               }

      default: {
                    GPS_state = 0;                         // reset state machine
                break;
               }
  }//switch


    if (GPS_Data_Expected) {                               // this part of code is executed only if GPS data is expected


      if (i < 45)
        GPS_Data[i] = tmp;                                 // moving the data received from UART to string GPS_Data[]
        i++;
// we're expecting responses like:
// "$GPSACP: 134720.500,4448.0464N,02027.1986E,4.8,-38.2,2,184.18,0.86,0.46,070709,03";
// data after 'W' or 'E' is of no use for our program
        if(tmp == 'W' || tmp == 'E'){
             j = i-1;
             GPS_Data_Ready = 1;                           // indicates that the data that we need for our calculation is received
   }
  }
  PIR1.RCIF = 0;                                           // clearing of the interuppt flag manually
 }
}



// send ATC command
void send_atc(const char *s)
{
// send command string
   while(*s) {
      UART1_Write(*s++);
   }
// terminate command with CR
   UART1_Write(0x0D);
}

// displaying your location on the world map
void Display_Cursor(signed int lat, signed int lon) {
  unsigned char latitude_y, longitude_x;

// latitude and longitude scaling for 128x64 display:
// latitude: input range is -90 to 90 degrees
// longitude: input range is -180 to 180 degrees
  latitude_y = ((61*(90 - lat))/180) + 1;
  longitude_x = ((125*(lon + 180))/360) + 1;

// cursor drawing:
  Glcd_Image(world_bmp);
  wait();
  Glcd_Dot(longitude_x,latitude_y,2);                      // central dot
  Glcd_Dot(longitude_x-1,latitude_y,2);                    // left dot
  Glcd_Dot(longitude_x+1,latitude_y,2);                    // right dot
  Glcd_Dot(longitude_x,latitude_y-1,2);                    // upper dot
  Glcd_Dot(longitude_x,latitude_y+1,2);                    // lower dot
  wait();

}



void main() {

  ANSEL  = 0;                                              // all pins as digital I/Os
  ANSELH = 0;
  C1ON_bit = 0;                                            // disable comparators
  C2ON_bit = 0;
  TRISE.B0 = 1;

  GPS_OK = 0;
  GPS_Data_Expected = 0;

  PIE1.RCIE = 1;
  PIR1.RCIF = 0;
  INTCON.PEIE = 1;

  UART1_Init(19200);                                       // initialize USART module

// set RTS pin to zero, we will use only RX i TX
  TRISC.B2 = 0;
  PORTC.B2 = 0;
//

  GLCD_Init();                                             // init GLCD
  Glcd_Set_Font(font5x7, 5, 7, 32);
  Glcd_Fill(0x00);
  wait3();

// welcome screen
  Glcd_Write_Text(CopyConst2Ram(GLCD_txt,glcd0), 6, 1, 1);
  Glcd_Write_Text(CopyConst2Ram(GLCD_txt,glcd1), 20, 3, 1);
  Glcd_Write_Text(CopyConst2Ram(GLCD_txt,glcd2), 6, 6, 1);
  Glcd_Write_Text(CopyConst2Ram(GLCD_txt,glcd3), 15, 7, 1);
//

  INTCON.GIE  = 1;                                         // allow all interrupts

  while(!PORTE.B0);                                        // when ready press RE0

  wait();
  Glcd_Fill(0x00);
  wait3();

// "please wait" screen
  Glcd_Write_Text(CopyConst2Ram(GLCD_txt,glcd4), 28, 3, 1);
  wait2();

// negotiate baud rade
  while(1) {
    send_atc(atc0);                                        // send "AT" string until gsm sets up its baud rade
    wait3();                                               // and gets it correctly
    if (GPS_OK)                                            // if gsm says "OK" on our baud rate than we've got it
    break;
  }

  GPS_OK = 0;
  send_atc(atc1);                                          // disable echo
  wait_GPS_OK();

  GPS_Data_Expected = 1;
  GPS_Data_Ready = 0;

// requesting the GPS data
  while(!GPS_Data_Ready){
  memset(GPS_Data, 0, 50);                                 // clear array of chars
  i = 0;
  j = 0;
  GPS_OK = 0;
  send_atc(atc2);                                          // request for your location details (latitude, longitude ... etc.)
  wait_GPS_OK();
  wait();
}

// process received data
  latitude = (GPS_data[j-21]-48)*10 + (GPS_data[j-20]-48);

  if(GPS_Data[j-12] == 'S') {
              latitude = 0 - latitude;                     // if the latitude is in the South direction it has minus sign
            }

  longitude = (GPS_data[j-10]-48)*100 + (GPS_data[j-9]-48)*10 + (GPS_data[j-8]-48);
  if(GPS_Data[j] == 'W') {
              longitude = 0 - longitude;                   // if the longitude is in the West direction it has minus sign
            }
//

  Glcd_Fill(0x00);
  wait3();

// "your location found" screen
  Glcd_Write_Text(CopyConst2Ram(GLCD_txt,glcd5), 6, 3, 1);
  wait();

// display location on the world map
    while(1)  {
                Display_Cursor(latitude, longitude);       // display the cursor on the world map
              }

}

// world map
unsigned char const world_bmp[1024] = {
 255,129,  1,  1,  1,129,129,129,129,193,129,129,129,129,129,129,
 129,129,129,129,129,225,161,161, 97, 97,209,209,129, 49, 49,201,
 201,201,201, 97,205,205,129,137, 25, 57, 57, 57,121,249,249,249,
 249,249,253,253,121,121,113,  9,  9,  1,  1,  1,  1,  1,  1,  1,
   1,  1,  1, 17, 17,145,145,145,145,129,129,129,  1,  1,  1,  1,
   9, 73, 73, 73, 73,193, 65, 65,129,129,193,193,129,193,193,241,
 241,241,241,225,225,225,193,193,193,193,193,193,193,193,193,129,
 193,193,225,225,129,129,129,129,129,129,129,129,129,129,129,255,
 255,  1, 33, 17, 17, 15, 15, 15, 15, 15,  7,  7,  7,  7, 15, 15,
  31, 63, 63, 63, 63,255,255,255,255,255,255,255,255,251,251,240,
 240,240,240,226,252,252,249,249,250,240,240,  1,  1,  1,  1,  3,
   1,  1,  0,  0,  0,  0,  0,  2,  2,  0,  0,  0, 24, 24,224,224,
 224,224,244,239,239,255,255,255,255,255,255,255,255,255,254,254,
 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
  95, 95,  3,  3,  3,  3, 63, 15, 15,  3,  3,  3,  3,  3,  1,255,
 255,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0, 15, 63, 63,255,255,255,255,255, 63, 63, 63,
  63, 63, 63, 63,135,135,  1,  1,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,192,192,192,243,243,251,251,
 251,251,251,247,231,231,243,247,247,247,230,236,124,124,255,255,
 220, 60, 61, 61, 63,126,255,255,255,255,255,255,255,255,255,255,
 255,255,255,255,255,255,255,255,255, 59, 59,  3,  7,  3, 27, 12,
   7,  7,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,255,
 255,  0,  0,  0,  0,  1,  1,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  1,  3,  6,  6, 13,
  13, 13, 13, 17,242,242,242,242,240,224,224,192,192,192,192,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0, 31, 31, 31, 63, 63, 63, 63,
  63, 63,255,255,255,255,255,255,255,255,255,255,248,248,247,247,
  55,  3,  3,  3,  3,  0,  1,  1,  3,  3, 15, 15,  7,  0,  0,  1,
   1,  3,  3,239, 15, 15,  1,129,224,174, 46,128,  0,128,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,255,
 255,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  3, 63, 63,255,255,255,255,255,255,255,255,255,255,
 254,254, 12, 12,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  1,255,255,255,255,255,255,255,255,255, 63, 63,193,193,
 240,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  3,  3,  4,  9,129,193,192,225,224,226,224,242,
 227,227,228,228,  8,  8,  0,  0,  0,  0,  0,  0,  0,  0,  0,255,
 255,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,255,255,255, 31, 31, 15, 15, 15, 15,  1,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  3,  3, 15, 15, 15, 15, 15,  3,  3,  0,  0,  1,  1,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  7, 15, 15,  7,  7,  7,  7,  7,
  31, 31,127,127, 70, 70,  0,  0,  0,  0,  0,  0,208,208,  0,255,
 255,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,135,135,193, 64, 68,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
 128,128,128,128,128,128,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,128,128,128,128,128,128,  0,  0,  0,  0,  0,  0,128,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,255,
 255,240,240,240,240,248,248,248,248,248,248,248,248,248,252,252,
 252,252,252,252,252,252,252,252,252,252,252,252,252,252,252,252,
 252,252,252,252,254,254,255,255,255,252,252,248,248,248,248,248,
 248,248,248,248,248,248,252,252,252,254,254,254,254,254,255,255,
 255,255,255,255,255,255,255,255,255,255,255,255,254,254,255,255,
 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
 255,255,255,255,255,255,255,255,255,250,250,250,216,216,248,255
};